@@ -25,7 +25,7 @@ static jfieldID  gladeFID;
 static jmethodID connectMID;
 
 jclass getClass_GladeXMLException(JNIEnv *env) {
-    static jclass cls = NULL;
+    jclass cls = NULL;
     if (cls == NULL)
         cls = (*env)->FindClass(env, "org/gnu/glade/GladeXMLException");
     return cls;
@@ -44,7 +44,9 @@ void libglade_log_handler (const gchar    *log_domain,
 {
     if (user_data) {
 	JNIEnv *env = (JNIEnv*)user_data;
-	(*env)->ThrowNew(env, getClass_GladeXMLException(env), message);
+	jclass cls = getClass_GladeXMLException(env);
+	(*env)->ThrowNew(env, cls, message);
+	(*env)->DeleteLocalRef(env, cls);
     }
 }
 
@@ -131,7 +133,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_glade_LibGlade_getNativeWidget (
 /*
  * int glade_xml_new_from_buffer(byte[], String)
  */
-JNIEXPORT void JNICALL 
+JNIEXPORT jobject JNICALL 
 Java_org_gnu_glade_LibGlade_glade_1xml_1new_1from_1buffer(
   JNIEnv *env, jobject o, jbyteArray buffer, jstring root) 
 {
@@ -159,7 +161,8 @@ Java_org_gnu_glade_LibGlade_glade_1xml_1new_1from_1buffer(
     if ((*env)->ExceptionCheck(env))
 	return;
 	
-    (*env)->SetObjectField(env, o, gladeFID, getHandleFromPointer(env, xml));
+    //(*env)->SetObjectField(env, o, gladeFID, getHandleFromPointer(env, xml));
+    return getHandleFromPointer(env, xml);
 }
 
 /*
@@ -168,7 +171,7 @@ Java_org_gnu_glade_LibGlade_glade_1xml_1new_1from_1buffer(
  * so that libglade's internals end up constructing widgets
  * with path references correctly
  */
-JNIEXPORT void JNICALL 
+JNIEXPORT jobject JNICALL 
 Java_org_gnu_glade_LibGlade_glade_1xml_1new(
   JNIEnv *env, jobject o, jstring file, jstring root) 
 {
@@ -189,7 +192,8 @@ Java_org_gnu_glade_LibGlade_glade_1xml_1new(
     if ((*env)->ExceptionCheck(env))
        return;
        
-    (*env)->SetObjectField(env, o, gladeFID, getHandleFromPointer(env, xml));
+    //(*env)->SetObjectField(env, o, gladeFID, getHandleFromPointer(env, xml));
+    return getHandleFromPointer(env, xml);
 }
 
 
@@ -216,8 +220,8 @@ JNIEXPORT void JNICALL Java_org_gnu_glade_LibGlade_initIDs (
     JNIEnv *env, jclass cls)
 {
     gladeFID = (*env)->GetFieldID(env, cls, "gladeHandle", 
-                                  "Lorg/gnu/glib/Handle;");
+                                  "Lorg/gnu/javagnome/Handle;");
 
     connectMID = (*env)->GetMethodID(env, cls, "connect",
-                                     "(Ljava/lang/String;Lorg/gnu/glib/Handle;Ljava/lang/String;Ljava/lang/String;Lorg/gnu/glib/Handle;Z)V");
+                                     "(Ljava/lang/String;Lorg/gnu/javagnome/Handle;Ljava/lang/String;Ljava/lang/String;Lorg/gnu/javagnome/Handle;Z)V");
 }
